# Making more adons

## Prerequisites

- [You know what you are doing.](1000.abstract.md)
- [adon is installed.](1100.installing.md)
- _optional_ [You went for a quick start.](1200.quickstart.md)
- [You read the tutorial: Making eves](../../eve/doc/tutorial1-forming-eves.md).
- You created a new elioSin theme called `genesis`.
- You're running `gulp` in its root folder.
- You are comely and cunning and corruptible.

## Overview

`adons` - in principle - work like this:

- `adon`s extend jQuery.
- Each `adon` gets its own js file so it can be imported singularly.
- The `adon` sets a className on the tag.
- The `adon` gets a scss file to style the adon's className. _optional_
- `adon.js` is referenced by the HTML page.
- `adon.scss` is referenced by `theme.scss` (or `judge.scss` if you don't want it in your theme).

## Step 1: Create an adon.js file

```scss
// YourApp/js/adonClipBoard.js
jQuery.fn.extend({
  adonClipBoard: function() {
    return this.each(function() {
      var $tag = $(this);
      tag_name = $tag.prop("tagName");
      $tag.addClass("adonClipBoard");
    });
  }
});
```

- Choose a name for your adon.
- We will use **ClipBoard** for this tutorial.
- In the `js` folder, create a file called `adonClipBoard.js`.
- Open `adonClipBoard.js` in your editor. Add the code shown.

Notice that we are adding a class. This class should be unique. It's only for your adon.

- NB: This is a good starting template for any `adon` ...

_elioSinners: refrain from using "adonXXX" for anything that does not affect every tag, so future users of your theme don't suffer from false expectations._

## Step 2: Putting `adon` to work

So far we have added a file, but it does nothing useful. Time to put `adon` to work for his sin. Let's add some code so that when someone clicks a tag, it copies the text in the tag into the computer's clipboard.

```javascript
// YourApp/js/adonClipBoard.js
jQuery.fn.extend({
  adonClipBoard: function() {
    return this.each(function() {
        var $tag = $(this);
        tag_name = $tag.prop("tagName");
        $tag.addClass("adonClipBoard");
        $tag.click(function() {
        var $temp = $("<input>");
        $("body").append($temp);
        $temp.val($tag.text()).select();
        document.execCommand("copy");
        $temp.remove();
        $tag.fadeOut(100).fadeIn(100);
      })
    });
  }
});
```

### Write the jQuery code

- Open `js/adonClipBoard.js`

- Insert the code shown. This is no jQuery tutorial, but this is what the script does:

  - Makes a temporary `input` tag.
  - Puts the text in the tag you clicked on into the `input` tag.
  - Executes a `copy` command.
  - Removes the temporary `input` tag.
  - Provides visual feedback to the user so they know something happened..

### Reference the jQuery code

```html
// YourApp/index.html
<script src="js/adonClipBoard.js"></script>
```

- Open the `index.html` file.
- Scroll to the bottom of the page, and find the `script` tag which references `js/main.js`
- Add a reference to _your_ adon, as shown.

### Installing the adon

```javascript
// YourApp/js/main.js
$(document).ready(function() {
  $("blockquote").adonClipBoard();
});
```

- Open `js/main.js`.
- Install the `adon` by plugging it into the `blockquote` tag as shown.
- This will make it easy for users to copy and paste your blockquotes.

Now try it! Click on a `blockquote` tag. Try pasting the content somewhere.

This is all great, but how is the user to _know_ about this feature?

Next we will work on the UI.

## Step 3: Theme the UI

Let's create an icon so the user knows about the copy feature.

### Adon the Icon

```javascript
// elioSin/js/adonClipBoard.js
var $but = $("<a><i class='fas fa-copy'></i></a>");
$but.addClass("adonClipBoardButton");
$but.master = $tag;
$tag.bar = $but;
$tag.append($but);
/* ^^^^^^ Add this */
$tag.click(function() { "..." })
```

- Open up the `js/adonClipBoard.js` again.
- Insert the code shown. This is no jQuery tutorial, but this is what the script does:

  - Dynamically creates a `i` tag.
  - Add the Font Awesome copy icon using the `fa-icon` class.
  - Appends the `i` tag to the tags where this plugin is being installed.

### Install Font Awesome

```html
<!-- YourApp/index.html -->
<link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.0.13/css/all.css" integrity="sha384-DNOHZ68U8hZfKXOrtjWvjxusGo9WQnrNx2sqG0tfsghAvtVlRW3tvkXWZh58N9jp" crossorigin="anonymous">
```

- Open up the `index.html` file again.
- Add the link to the Font Awesome library inside your `head` tag somewhere.

### Theme the Icon

```sass
// YourApp/stylesheets/adon/adonClipBoard.scss
.adonClipBoard  .adonClipBoardButton {
  float: left;
  cursor: pointer;
}
```

Next we will add some SASS to fix the icon to the top of the `blockquote`.

- Create a new folder in `stylesheets` called `adon`.
- Create a new file called `stylesheets/adon/adonClipBoard.scss`.
- Add the code shown to style the icon.

### Reference the `adon` stylesheet

Finally we need to make sure the adonClipBoard style is added to our `genesis` theme.

```sass
// YourApp/stylesheets/theme.scss
@import "adon/adonClipBoard";
```

- Open the `stylesheets/theme.scss` file.
- Add the `@import` line shown.

You now have a "copy" icon sitting to the left of your `blockquote` and your `adon` is created.

You can rest.
